# -*- coding: utf-8 -*-

import os
import base64
import datetime
import json
import xlwt
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.core.exceptions import MultipleObjectsReturned
from django.core.urlresolvers import reverse, reverse_lazy
from django.db.models import Q
from django.http import HttpResponse, Http404
from django.views.generic.base import TemplateView, View
from djappypod.response import OdtTemplateResponse

from web.utils.appypod import get_photo_appy
from acomea.soap2.soap import MyAcomeaSoapClient
from acomea.my_acomea.models import FileGestioni
from acomea.my_acomea.utils.views import get_fondi_dettagli_ripartizione_pie_dataset
from acomea.profilo.models import RichiestaAttivazioneCliente


class MyAcomeAMixin(TemplateView):
    soap_client = MyAcomeaSoapClient()
    partecipazione = None
    profilo, rac = None, None
    cliente = None
    codice_fiscale = ''
    file_name, file_title = 'myacomea', 'myacomea'
    data_giro_key = 'data_giro_my_acomea'
    data_valo_key = 'data_valo_my_acomea'
    request = None
    user_cliente = None

    def get(self, request, *args, **kwargs):
        if 'type' in self.request.GET:
            response = HttpResponse(mimetype='application/ms-excel')
            response['Content-Disposition'] = 'attachment; filename={0}.xls'.format(self.file_name)
            wb = self.get_xls(**kwargs)
            wb.save(response)
            return response
        return super(MyAcomeAMixin, self).get(request, *args, **kwargs)

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.partecipazione = kwargs.get('partecipazione', None)
        self.cliente = request.session.get("cliente_my_acomea", None)
        self.codice_fiscale = request.session.get("codice_fiscale", "")
        self.rac = self.get_rac_cliente()
        self.profilo = self.get_profilo_cliente()
        self.user_cliente = self.get_user_cliente()

        if not self.cliente:
            self.cliente = self.get_cliente()
        self.data_giro = request.session.get(
            self.data_giro_key, self.get_data_giro(request)
        )
        self.data_valo = request.session.get(
            self.data_valo_key, self.get_data_valo(request)
        )

        return super(MyAcomeAMixin, self).dispatch(request, *args, **kwargs)

    def get_rac_cliente(self):
        try:
            return RichiestaAttivazioneCliente.objects.get(codice_fiscale=self.codice_fiscale)
        except RichiestaAttivazioneCliente.DoesNotExist:
            if hasattr(self.request.user, 'profilo') and self.request.user.profilo.is_acomea:
                return self.request.user.profilo.rac
            return ''
        except MultipleObjectsReturned:
            return ''

    def get_profilo_cliente(self):
        if self.rac and hasattr(self.rac, 'profilo'):
            return self.rac.profilo
        elif hasattr(self.request.user, 'profilo') and self.request.user.profilo.is_acomea:
            return self.request.user.profilo
        else:
            return ''

    def get_user_cliente(self):
        if self.profilo and hasattr(self.profilo, 'user'):
            return self.profilo.user
        elif hasattr(self.request.user, 'profilo') and self.request.user.profilo.is_acomea:
            return self.request.user
        else:
            return ''

    def get_cliente(self):
        if self.codice_fiscale:
            cliente = self.soap_client.get_cliente(self.codice_fiscale)
            self.request.session["cliente_my_acomea"] = cliente
            return cliente
        return None

    def get_data_giro(self, request):
        data_giro = self.soap_client.get_DataGiro()
        request.session[self.data_giro_key] = data_giro
        return data_giro

    def get_data_valo(self, request):
        data_valo = self.soap_client.get_DataValo()
        request.session[self.data_valo_key] = data_valo
        return data_valo

    @property
    def is_cliente_full(self):
        if self.profilo and self.rac:
            profilo, rac = self.profilo, self.rac
            return (
                (
                    profilo.is_cliente and rac.stato in ('9', '6') and
                    profilo.abilitato and rac.codice_fiscale
                ) or profilo.is_acomea
            )
        return None

    def get_codice_fiscale(self):
        user = self.request.user
        if 'codice_fiscale' in self.request.session:
            cf = self.request.session['codice_fiscale']
        elif user.is_active and hasattr(user, 'profilo') and hasattr(user.profilo, 'rac'):
            cf = user.profilo.rac.codice_fiscale
        else:
            cf = ''
        if cf:
            self.request.session['codice_fiscale'] = cf
        return cf

    @staticmethod
    def get_period_interval(filter):
        today = datetime.date.today()
        date_end = today
        if filter == 'ultimo_mese':
            date_begin = today + relativedelta(months=-1)
        elif filter == 'ultimi_3_mesi':
            date_begin = today + relativedelta(months=-3)
        elif filter == 'ultimo_anno':
            date_begin = today + relativedelta(years=-1)
        else:
            date_begin = datetime.date(1970, 1, 1)
        return date_begin, date_end


class OperazioniEffettuateMixin(MyAcomeAMixin):

    def get_context_data(self, **kwargs):
        context = super(OperazioniEffettuateMixin, self).get_context_data(**kwargs)
        filter = self.request.GET.get('period' or None)
        date_begin, date_end = self.get_period_interval(filter)
        rapporto = int(self.request.GET.get('rapporto', 0))
        if self.is_cliente_full and self.codice_fiscale:
            elenco_movimenti = self.soap_call(self.partecipazione, self.codice_fiscale, date_begin, date_end, rapporto)
            elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
                self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
            rapporti = [rapporto for rapporto in elenco_rapporti]
            url_dettagli = {
                'FONDICOMUNI': reverse(
                    'myacomea:fondi_operazioni_effettuate', kwargs={'partecipazione': self.partecipazione}
                ),
                'GESTIONI': reverse(
                    'myacomea:gestioni_operazioni_effettuate', kwargs={'partecipazione': self.partecipazione}
                )
            }
            context.update({
                'prodotto': self.prodotto,
                'partecipazione': self.partecipazione,
                'elenco_movimenti': elenco_movimenti,
                'rapporti': rapporti,
                'url_dettaglio': url_dettagli[self.prodotto],
                'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
                'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
                'detail_url': reverse('myacomea:fondi_operazioni_effettuate_dettaglio'),
                'modal_id': "modal-{}".format(self.partecipazione)
            })
        return context

    def soap_call(self, partecipazione, codice_fiscale, date_begin, date_end, rapporto):
        raise NotImplementedError


class MyAcomea(MyAcomeAMixin):
    template_name = "my_acomea/my_acomea/riepilogo.html"
    lista_prodotti = ('TUTTO', 'FONDICOMUNI', 'GESTIONI', 'CONSULENZA')
    intestatario_fields = ['fondi_rapporti_intestatario', 'gestioni_contratti_intestatario']
    cointestatario_fields = ['fondi_rapporti_cointestatario', 'gestioni_contratti_cointestatario']

    def get_context_data(self, **kwargs):
        context = super(MyAcomea, self).get_context_data(**kwargs)
        if self.is_cliente_full and self.codice_fiscale:
            dati_riepilogo = {}
            for prodotto in self.lista_prodotti:
                dati_riepilogo[prodotto] = self.soap_client.get_AR_Riepilogo(
                    prodotto, self.partecipazione, self.codice_fiscale
                )

            context.update({
                'partecipazione': self.partecipazione,
                'dati_riepilogo': dati_riepilogo,
                'data_posizione': dati_riepilogo['TUTTO']['data_posizione'],
                'url_dettagli':  reverse(
                    'myacomea:my_acomea_dettagli', kwargs={'partecipazione': self.partecipazione}
                ),
                'is_intestatario': bool(max([self.cliente[field] for field in self.intestatario_fields])),
                'is_cointestatario': bool(max([self.cliente[field] for field in self.cointestatario_fields])),
                'data_giro': self.data_giro,
                'data_valo': self.data_valo
            })
        return context


class MyAcomeaDettagli(MyAcomeAMixin):
    template_name = "my_acomea/my_acomea/dettagli.html"

    def get_context_data(self, **kwargs):
        context = super(MyAcomeaDettagli, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))

        product_exist_keys = {
            'intestatario': {
                'FONDICOMUNI': 'has_fondi_comuni_intestatario',
                'GESTIONI': 'has_gestioni_intestatario',
            },
            'cointestatario': {
                'FONDICOMUNI': 'has_fondi_comuni_cointestatario',
                'GESTIONI': 'has_gestioni_cointestatario',
            }
        }

        if self.is_cliente_full and self.codice_fiscale:
            dati_dettagli = []
            for prodotto in ["FONDICOMUNI", "GESTIONI"]:
                session_key = "datasets_{}_{}_{}".format(prodotto, self.partecipazione, rapporto)
                datasets = self.request.session.get(session_key, None)
                if self.request.session.get("codice_fiscale", None) or not datasets:
                    dati_ripartizione = self.soap_client.get_AR_Ripartizione(
                        prodotto, self.partecipazione, self.codice_fiscale, rapporto
                    )
                    datasets = {
                        'dati_riepilogo': self.soap_client.get_AR_Riepilogo(
                            prodotto, self.partecipazione, self.codice_fiscale, rapporto
                        ),
                        'dati_dettaglio': self.soap_client.get_AR_SintesiRapporti(
                            prodotto, self.partecipazione, self.codice_fiscale, rapporto
                        ),
                        'dati_ripartizione': dati_ripartizione,
                        'dataset_dati_ripartizione': get_fondi_dettagli_ripartizione_pie_dataset(
                            dati_ripartizione, self.partecipazione, prodotto
                        )
                    }
                    self.request.session[session_key] = datasets
                dati_dettagli.append({
                    'partecipazione': self.partecipazione,
                    'prodotto': prodotto,
                    'has_prodotto': self.cliente[product_exist_keys[self.partecipazione.lower()][prodotto]],
                    'datasets': datasets,
                    'titolo': 'Fondi comuni' if prodotto == 'FONDICOMUNI' else 'Gestioni',
                    'show_modal': False,
                })
            context.update({
                'dati_dettagli': dati_dettagli,
                'data_giro': self.data_giro,
                'data_valo': self.data_valo,
            })
            return context


class ProdottoDettaglioPartecipazione(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/dettaglio_partecipazione.html"
    show_modal = True
    modal_url = None

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["/my_acomea/includes/_dettaglio_dati.html"]
        return super(ProdottoDettaglioPartecipazione, self).get_template_names()

    def get_context_data(self, **kwargs):
        context = super(ProdottoDettaglioPartecipazione, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))
        session_key = "datasets_{}_{}_{}".format(self.prodotto, self.partecipazione, rapporto)
        datasets = self.request.session.get(session_key, None)
        if self.prodotto == 'FONDICOMUNI':
            url_dettaglio = reverse('myacomea:fondi_dettaglio', kwargs={'partecipazione': self.partecipazione})
        else:
            url_dettaglio = reverse('myacomea:gestioni_dettaglio', kwargs={'partecipazione': self.partecipazione})

        if self.is_cliente_full and self.codice_fiscale:
            if self.request.session.get("codice_fiscale", None) or not datasets:
                dati_ripartizione = self.soap_client.get_AR_Ripartizione(
                    self.prodotto, self.partecipazione, self.codice_fiscale, rapporto
                )
                datasets = {
                    'dati_riepilogo': self.soap_client.get_AR_Riepilogo(
                        self.prodotto, self.partecipazione, self.codice_fiscale, rapporto
                    ),
                    'dati_dettaglio': self.soap_client.get_AR_SintesiRapporti(
                        self.prodotto, self.partecipazione, self.codice_fiscale, rapporto
                    ),
                    'dati_ripartizione': dati_ripartizione,
                    'dataset_dati_ripartizione': get_fondi_dettagli_ripartizione_pie_dataset(
                        dati_ripartizione, self.partecipazione, self.prodotto
                    )
                }
                self.request.session[session_key] = datasets
        elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
            self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
        elenco_rapporti = [rapp for rapp in elenco_rapporti]
        rapporti_visibili = list(set([dettaglio['rapporto'] for dettaglio in datasets['dati_dettaglio']]))
        is_rapporto_singolo = True if rapporto else len(rapporti_visibili) == 1
        if is_rapporto_singolo and self.prodotto == 'FONDICOMUNI':
            cod_rapporto_singolo = rapporti_visibili[0] if is_rapporto_singolo else None
            prodotto = (prodotto for prodotto in self.cliente['prodotti'] if prodotto['tipo_prodotto'] == 'FC').next()
            rapporto_singolo = (rapporto for rapporto in prodotto['rapporti'] if
                                int(rapporto['codice_rapporto']) == int(cod_rapporto_singolo)).next()
            context.update({'rapporto_singolo': rapporto_singolo})
        context.update({
            'partecipazione': self.partecipazione,
            'prodotto': self.prodotto,
            'cliente': self.cliente,
            'datasets': datasets,
            'rapporti': elenco_rapporti,
            'is_rapporto_singolo': is_rapporto_singolo,
            'url_dettaglio': url_dettaglio,
            'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
            'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
            'data_giro': datasets['dati_riepilogo']['data_nav'],
            'data_valo': datasets['dati_riepilogo']['data_posizione'],
            'show_modal': self.show_modal,
            'modal_url': self.modal_url,
            'modal_id': "modal-{}".format(self.partecipazione),
        })
        return context


class FondiDettaglioPartecipazione(ProdottoDettaglioPartecipazione):
    prodotto = 'FONDICOMUNI'
    intestatario_rapporti_key = 'fondi_rapporti_intestatario'
    cointestatario_rapporti_key = 'fondi_rapporti_cointestatario'
    modal_url = reverse_lazy('myacomea:fondi_dettaglio_modale')


class FondiDettaglioModale(TemplateView):
    template_name = "/my_acomea/prodotto/fondi/dettaglio_modale.html"
    soap_client = MyAcomeaSoapClient()

    def get_context_data(self, **kwargs):
        context = super(FondiDettaglioModale, self).get_context_data(**kwargs)
        rapporto = self.request.GET.get('rapporto')
        deposito = self.request.GET.get('deposito')
        fondo = self.request.GET.get('fondo')
        tc = self.request.GET.get('tc')
        if tc:
            tc = tc.encode('utf-8')
        dettaglio = self.soap_client.get_AR_DEPFON(rapporto, deposito, fondo, tc)
        context.update({'dettaglio': dettaglio})
        return context


class FondiAnalisiTemporale(MyAcomeAMixin):
    template_name = "/my_acomea/empty.html"
    prodotto = 'FONDICOMUNI'

    def get_context_data(self, **kwargs):
        request = self.request
        today = datetime.date.today()
        context = super(FondiAnalisiTemporale, self).get_context_data(**kwargs)

        for key in ["data_dal", "data_al", "data_dal_confronto"]:
            key_value = None
            if key in request.GET:
                try:
                    key_value = datetime.datetime.strptime(self.request.GET.get(key), "%d/%m/%Y").date()
                except ValueError:
                    pass
            request.session["analisi_temporale_{}".format(key)] = key_value
        if self.user_cliente and self.user_cliente.is_active and self.profilo and self.rac:
            data_dal = request.session['analisi_temporale_data_dal']
            data_dal_confronto = request.session['analisi_temporale_data_dal_confronto']
            data_al = request.session['analisi_temporale_data_al']

            if not data_dal and not data_dal_confronto and not data_al:
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'Devi inserire almeno una data'})

            elif data_dal and (data_dal_confronto or data_al):
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'Devi selezionare le date solamente per un form'})

            elif data_dal_confronto and not data_al:
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'Devi selezionare entrambe le date'})

            elif not data_dal_confronto and data_al:
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'Devi selezionare entrambe le date'})

            elif data_dal_confronto and data_al and data_dal_confronto >= data_al:
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'La data di partenza non può essere maggiore od uguale a quella finale'})

            elif data_dal_confronto and data_dal_confronto >= today:
                self.template_name = "/my_acomea/prodotto/data_error.html"
                context.update({'message': 'La data iniziale non può essere maggiore od uguale a quella odierna'})

            elif data_dal:
                self.template_name = "/my_acomea/prodotto/tabs.html"
                if data_dal >= today:
                    context.update({'message': 'Hai inserito una data superiore o uguale ad oggi. '
                                               '<strong>'
                                               'I dati rappresentati si riferiscono all\'ultima data disponibile'
                                               '</strong>'
                                    })
                context.update({
                    'today': request.session['analisi_temporale_data_dal'],
                    'url_intestatario':  reverse(
                        'myacomea:analisi_temporale_dettaglio', kwargs={'partecipazione': 'INTESTATARIO'}
                    ),
                    'url_cointestatario': reverse(
                        'myacomea:analisi_temporale_dettaglio', kwargs={'partecipazione': 'COINTESTATARIO'}
                    ),
                })

            elif data_dal_confronto and data_al:
                self.template_name = "/my_acomea/prodotto/tabs.html"
                if data_al >= today:
                    context.update({'message': 'La data finale si riferise all\'ultima data disponibile'})
                context.update({
                    'today': request.session['analisi_temporale_data_dal'],
                    'url_intestatario':  reverse(
                        'myacomea:analisi_temporale_confronto', kwargs={'partecipazione': 'INTESTATARIO'}
                    ),
                    'url_cointestatario': reverse(
                        'myacomea:analisi_temporale_confronto', kwargs={'partecipazione': 'COINTESTATARIO'}
                    ),
                })

        return context


class FondiAnalisiTemporaleDettaglio(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/dettaglio_partecipazione.html"
    prodotto = 'FONDICOMUNI'
    intestatario_rapporti_key = 'fondi_rapporti_intestatario'
    cointestatario_rapporti_key = 'fondi_rapporti_cointestatario'
    show_modal = False

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["/my_acomea/includes/_dettaglio_dati.html"]
        return super(FondiAnalisiTemporaleDettaglio, self).get_template_names()

    def get_context_data(self, **kwargs):
        context = super(FondiAnalisiTemporaleDettaglio, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))
        data_dal = self.request.session['analisi_temporale_data_dal']
        session_key = "datasets_storico_{}_{}_{}_{}".format(
            self.prodotto, self.partecipazione, rapporto, data_dal.strftime('%Y%m%d')
        )
        datasets = self.request.session.get(session_key, None)
        url_dettaglio = reverse('myacomea:analisi_temporale_dettaglio', kwargs={'partecipazione': self.partecipazione})
        if self.is_cliente_full and self.codice_fiscale:
            if self.request.session.get("codice_fiscale", None) or not datasets:
                posizione_storica = self.soap_client.get_AR_PosizioneStorica(
                    self.prodotto, self.partecipazione, self.codice_fiscale, data_dal, rapporto
                )
                dataset_dati_ripartizione = get_fondi_dettagli_ripartizione_pie_dataset(
                    posizione_storica['ripartizione'], self.partecipazione, self.prodotto,
                )
                datasets = {
                    'dati_riepilogo': posizione_storica['riepilogo'],
                    'dati_dettaglio': posizione_storica['sintesi'],
                    'dati_ripartizione': posizione_storica['ripartizione'],
                    'dataset_dati_ripartizione': dataset_dati_ripartizione,
                }
                self.request.session[session_key] = datasets
            elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
                self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
            rapporti = [rapp for rapp in elenco_rapporti]
            rapporti_visibili = list(set([dettaglio['rapporto'] for dettaglio in datasets['dati_dettaglio']]))
            is_rapporto_singolo = True if rapporto else len(rapporti_visibili) == 1
            if is_rapporto_singolo:
                cod_rapporto_singolo = rapporti_visibili[0] if is_rapporto_singolo else None
                prodotto = (prodotto for prodotto in self.cliente['prodotti'] if prodotto['tipo_prodotto'] == 'FC').next()
                rapporto_singolo = (rapporto for rapporto in prodotto['rapporti'] if
                                    int(rapporto['codice_rapporto']) == int(cod_rapporto_singolo)).next()
                context.update({'rapporto_singolo': rapporto_singolo})
            context.update({
                'partecipazione': self.partecipazione,
                'prodotto': self.prodotto,
                'datasets': datasets,
                'rapporti': rapporti,
                'is_rapporto_singolo': is_rapporto_singolo,
                'url_dettaglio': url_dettaglio,
                'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
                'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
                'data_giro': datasets['dati_riepilogo']['data_nav'],
                'data_valo': datasets['dati_riepilogo']['data_posizione'],
                'show_modal': self.show_modal,
            })
        return context


class FondiAnalisiTemporaleConfronto(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/confronto.html"
    prodotto = 'FONDICOMUNI'
    intestatario_rapporti_key = 'fondi_rapporti_intestatario'
    cointestatario_rapporti_key = 'fondi_rapporti_cointestatario'

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["/my_acomea/includes/_dettaglio_dati_confronto.html"]
        return super(FondiAnalisiTemporaleConfronto, self).get_template_names()

    def get_context_data(self, **kwargs):
        context = super(FondiAnalisiTemporaleConfronto, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))
        data_dal = self.request.session['analisi_temporale_data_dal_confronto']
        data_al = self.request.session['analisi_temporale_data_al']
        session_key = "confronto_{}_{}_{}_{}_{}".format(
            self.prodotto, self.partecipazione, rapporto, data_dal.strftime('%Y%m%d'), data_al.strftime('%Y%m%d')
        )
        confronto = self.request.session.get(session_key, None)
        url_dettaglio = reverse('myacomea:analisi_temporale_confronto', kwargs={'partecipazione': self.partecipazione})
        if self.is_cliente_full and self.codice_fiscale:
            if self.request.session.get("codice_fiscale", None) or not confronto:
                confronto = self.soap_client.get_AR_Confronto(
                    self.partecipazione, self.codice_fiscale, data_dal, data_al, rapporto
                )
                self.request.session[session_key] = confronto
            elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
                self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
            rapporti = [rapporto for rapporto in elenco_rapporti]
            context.update({
                'partecipazione': self.partecipazione,
                'confronto': confronto,
                'rapporti': rapporti,
                'url_dettaglio': url_dettaglio,
                'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
                'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
                'data_dal': data_dal,
                'data_al': data_al,
                'detail_url': reverse('myacomea:fondi_operazioni_effettuate_dettaglio'),
                'modal_id': "modal-{}".format(self.partecipazione),
                'data_giro': self.data_giro,
                'data_valo': data_al if data_al < datetime.date.today() else self.data_valo
            })
        return context


class FondiOperazioniInCorso(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/operazioni_in_corso.html"

    def get_context_data(self, **kwargs):
        context = super(FondiOperazioniInCorso, self).get_context_data(**kwargs)
        if self.is_cliente_full and self.codice_fiscale:
            context.update({
                'operazioni_internet': self.soap_client.get_operazioni_in_corso_web(self.codice_fiscale),
                'operazioni_da_valorizzare': self.soap_client.get_operazioni_in_corso_contra(self.codice_fiscale),
                'url_eliminazione': reverse('myacomea:fondi_eliminazione_operazione_in_corso'),
                'detail_url': reverse('myacomea:fondi_operazioni_in_corso_dettaglio'),
                'modal_id': "modal-{}".format(self.partecipazione),
            })
        return context


class FondiOperazioniEffettuate(OperazioniEffettuateMixin):
    template_name = "/my_acomea/prodotto/operazioni_effettuate_fondi.html"
    prodotto = 'FONDICOMUNI'
    intestatario_rapporti_key = 'fondi_rapporti_intestatario'
    cointestatario_rapporti_key = 'fondi_rapporti_cointestatario'
    file_name = 'fondi_operazioni_effettuate'
    file_title = 'Fondi operazioni effettuate'

    def soap_call(self, partecipazione, codice_fiscale, date_begin, date_end, rapporto):
        elenco_movimenti = self.soap_client.get_AR_MovimentiFC(
            partecipazione, codice_fiscale, date_begin, date_end, rapporto
        )
        return elenco_movimenti

    def get_xls(self, **kwargs):
        context = self.get_context_data(**kwargs)
        wb = xlwt.Workbook()
        xlwt.add_palette_colour("custom_colour", 0x21)
        wb.set_colour_RGB(0x21, 230, 81, 18)
        title = xlwt.easyxf('font: bold on, height 480;')
        subtitle = xlwt.easyxf('font: height 380;')
        thead = xlwt.easyxf('font: bold on;''pattern: pattern solid, fore_colour custom_colour;')
        data = xlwt.easyxf(num_format_str='D-MMM-YY')
        quote = xlwt.easyxf(num_format_str="#,###0.000")
        importo = xlwt.easyxf(num_format_str="#,##0.00")
        ws = wb.add_sheet('Test Sheet')
        ws.write(0, 0, self.file_title, title)
        ws.write(1, 0, self.partecipazione, subtitle)
        ws.write(2, 0, 'I dati rappresentati sono puramente indicativi e non devono essere considerati di rilevanza contabile')
        ws.write(4, 0, 'Data', thead)
        ws.write(4, 1, 'Cliente', thead)
        ws.write(4, 2, 'Rapporto', thead)
        ws.write(4, 3, 'Contratto', thead)
        ws.write(4, 4, 'Fondo', thead)
        ws.write(4, 5, 'Causale', thead)
        ws.write(4, 6, 'T.Contr.', thead)
        ws.write(4, 7, 'Quote', thead)
        ws.write(4, 8, 'Imp.Netto', thead)
        for i, movimento in enumerate(context['elenco_movimenti']):
            i=i+5
            ws.write(i, 0, movimento['Data'], data)
            ws.write(i, 1, movimento['CodCliente'])
            ws.write(i, 2, movimento['CodRapporto'])
            ws.write(i, 3, movimento['NumContratto'])
            ws.write(i, 4, movimento['DescrFondo'])
            ws.write(i, 5, movimento['CausaleDescr'])
            ws.write(i, 6, movimento['TipoContratto'])
            ws.write(i, 7, movimento['Quote'], quote)
            ws.write(i, 8, movimento['ImportoNetto'], importo)
        return wb

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["/my_acomea/includes/_dettaglio_fondi_operazioni_effettuate.html"]
        return super(FondiOperazioniEffettuate, self).get_template_names()


class FondiServiziAttivi(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/servizi_attivi.html"

    def get_context_data(self, **kwargs):
        context = super(FondiServiziAttivi, self).get_context_data(**kwargs)
        if self.is_cliente_full and self.codice_fiscale:
            context.update({
                'servizi_inseriti': self.soap_client.get_servizi_in_corso_web(self.codice_fiscale),
                'servizi_attivi': self.soap_client.get_servizi_attivi(self.codice_fiscale),
                'url_eliminazione': reverse('myacomea:fondi_eliminazione_servizio_inserito'),
                'url_revoca': reverse('myacomea:fondi_revoca_servizio_attivo'),
            })
        return context


class FondiOperazioniIncorsoDettaglio(TemplateView):
    template_name = "/my_acomea/prodotto/operazioni_incorso_dettaglio.html"

    def get_context_data(self, **kwargs):
        context = super(FondiOperazioniIncorsoDettaglio, self).get_context_data(**kwargs)
        beneficiario_bonifico = self.request.GET.get('beneficiario-bonifico')
        IBAN_bonifico = self.request.GET.get('IBAN-bonifico')
        importo_bonifico = self.request.GET.get('importo-bonifico')
        causale_bonifico = self.request.GET.get('causale-bonifico')
        context.update({
            'beneficiario_bonifico': beneficiario_bonifico,
            'IBAN_bonifico': IBAN_bonifico,
            'importo_bonifico': importo_bonifico,
            'causale_bonifico': causale_bonifico,
        })
        return context

class FondiOperazioniEffettuateDettaglio(TemplateView):
    template_name = "/my_acomea/prodotto/operazioni_effettuate_dettaglio.html"
    soap_client = MyAcomeaSoapClient()

    def get_context_data(self, **kwargs):
        context = super(FondiOperazioniEffettuateDettaglio, self).get_context_data(**kwargs)
        progr_caricamento = self.request.GET.get('progr-caricamento')
        progr_rec_aggancio = self.request.GET.get('progr-rec-aggancio')
        movimento = self.soap_client.get_AR_MovimentoDett(progr_caricamento, progr_rec_aggancio)
        context.update({
            'movimento': movimento,
            'progCaricamento': progr_caricamento,
            'progRecAggancio': progr_rec_aggancio,
        })
        return context


class FondiEliminazioneOperazioneInCorso(View):
    soap_client = MyAcomeaSoapClient()

    def get(self, request):
        numero_operazione = request.GET.get('numero_operazione', None)
        codice_fiscale = request.GET.get('codice_fiscale', None)
        result = False
        if numero_operazione and codice_fiscale:
            result = self.soap_client.annulla_operazione_in_corso_web(numero_operazione, codice_fiscale)
        return HttpResponse(json.dumps(result), content_type="application/json")


class FondiEliminazioneServizioInserito(View):
    soap_client = MyAcomeaSoapClient()

    def get(self, request):
        id_servizio = request.GET.get('id_servizio', None)
        codice_fiscale = request.GET.get('codice_fiscale', None)
        result = False
        if id_servizio and codice_fiscale:
            result = self.soap_client.annulla_servizio_web(id_servizio, codice_fiscale)
        return HttpResponse(json.dumps(result), content_type="application/json")


class FondiRevocaServizioAttivo(View):
    soap_client = MyAcomeaSoapClient()
    msg = "Revoca del servizio inoltrata correttamente"

    def get(self, request):
        id_servizio = request.GET.get('id_servizio', None)
        tipo_servizio = request.GET.get('tipo_servizio', None)
        simulazione, codice_fiscale = "S", None
        result = False

        if request.user.profilo.rac.is_cliente:
            codice_fiscale = request.user.profilo.rac.codice_fiscale
            simulazione = "N"
        elif request.user.profilo.rac.is_acomea:
            codice_fiscale = request.session.get('codice_fiscale', '')

        if id_servizio and tipo_servizio and codice_fiscale:
            result = self.soap_client.revoca_servizio_attivo(id_servizio, codice_fiscale, simulazione, tipo_servizio)
        return HttpResponse(json.dumps(result), content_type="application/json")


class GestioniDettaglioPartecipazione(ProdottoDettaglioPartecipazione):
    prodotto = "GESTIONI"
    intestatario_rapporti_key = 'gestioni_contratti_intestatario'
    cointestatario_rapporti_key = 'gestioni_contratti_cointestatario'
    show_modal = False


class GestioniOperazioniEffettuate(OperazioniEffettuateMixin):
    template_name = "/my_acomea/prodotto/operazioni_effettuate_gestioni.html"
    prodotto = 'GESTIONI'
    intestatario_rapporti_key = 'gestioni_contratti_intestatario'
    cointestatario_rapporti_key = 'gestioni_contratti_cointestatario'
    file_name = 'gestioni_operazioni_effettuate'
    file_title = 'Gestioni operazioni effettuate'

    def soap_call(self, partecipazione, codice_fiscale, date_begin, date_end, rapporto):
        elenco_movimenti = self.soap_client.get_AR_MovimentiGI(
            partecipazione, codice_fiscale, date_begin, date_end, rapporto
        )
        return elenco_movimenti

    def get_xls(self, **kwargs):
        context = self.get_context_data(**kwargs)
        wb = xlwt.Workbook()
        xlwt.add_palette_colour("custom_colour", 0x21)
        wb.set_colour_RGB(0x21, 230, 81, 18)
        title = xlwt.easyxf('font: bold on, height 480;')
        subtitle = xlwt.easyxf('font: height 380;')
        thead = xlwt.easyxf('font: bold on;''pattern: pattern solid, fore_colour custom_colour;')
        data = xlwt.easyxf(num_format_str='D-MMM-YY')
        ctv = xlwt.easyxf(num_format_str="#,##0.00")
        ws = wb.add_sheet('Test Sheet')
        ws.write(0, 0, self.file_title, title)
        ws.write(1, 0, self.partecipazione, subtitle)
        ws.write(2, 0, 'I dati rappresentati sono puramente indicativi e non devono essere considerati di rilevanza contabile')
        ws.write(4, 0, 'Data operazione', thead)
        ws.write(4, 1, 'Data valuta', thead)
        ws.write(4, 2, 'Rapporto', thead)
        ws.write(4, 3, 'Linea', thead)
        ws.write(4, 4, 'Causale', thead)
        ws.write(4, 5, 'Titolo (isin)', thead)
        ws.write(4, 6, 'Tipo Movimento', thead)
        ws.write(4, 7, 'Ctv', thead)
        for i, movimento in enumerate(context['elenco_movimenti']):
            i=i+5
            ws.write(i, 0, movimento['data_operazione'], data)
            ws.write(i, 1, movimento['data_valuta'], data)
            ws.write(i, 2, movimento['dossier'])
            ws.write(i, 3, movimento['linea'])
            ws.write(i, 4, movimento['causale'])
            ws.write(i, 5, movimento['titolo_isin'])
            ws.write(i, 6, movimento['tipo_movimento'])
            ws.write(i, 7, movimento['ctv'], ctv)
        return wb

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["/my_acomea/includes/_dettaglio_gestioni_operazioni_effettuate.html"]
        return super(GestioniOperazioniEffettuate, self).get_template_names()


class GestioniStoricoEstrattoConto(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/gestioni/storico_estratto_conto.html"
    intestatario_rapporti_key = 'gestioni_rapporti_intestatario'
    cointestatario_rapporti_key = 'gestioni_rapporti_cointestatario'

    def get_context_data(self, **kwargs):
        context = super(GestioniStoricoEstrattoConto, self).get_context_data(**kwargs)
        filter = self.request.GET.get('period' or None)
        date_begin, date_end = self.get_period_interval(filter)
        rapporto = int(self.request.GET.get('rapporto', 0))
        url_dettaglio = reverse(
            'myacomea:gestioni_storico_estratto_conto_dettaglio', kwargs={'partecipazione': self.partecipazione}
        )
        if self.is_cliente_full and self.codice_fiscale:
            rapporti_key = self.intestatario_rapporti_key if \
                self.partecipazione.lower() == "intestatario" else self.cointestatario_rapporti_key
            rapporti = ["0{}".format(rapp) for rapp in self.cliente[rapporti_key]] if \
                not rapporto else ["0{}".format(rapporto)]
            rapporti_filter = Q()
            for rapp in rapporti:
                rapporti_filter = rapporti_filter | Q(rapporto__contains=rapp)
            if date_begin and date_end:
                rapporti_filter = rapporti_filter & Q(date__range=[date_begin, date_end])
            rendiconti = FileGestioni.objects.filter(rapporti_filter).order_by('-date') if \
                rapporti else FileGestioni.objects.none()
            context.update({
                'partecipazione': self.partecipazione,
                'url_dettaglio': url_dettaglio,
                'rapporti': rapporti,
                'rendiconti': rendiconti,
                'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
                'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
            })
        return context

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["my_acomea/includes/_dettaglio_storico_estratto_conto.html"]
        return super(GestioniStoricoEstrattoConto, self).get_template_names()


class GestioniStoricoEstrattoContoDownload(MyAcomeAMixin):
    template_name = None
    intestatario_rapporti_key = 'gestioni_rapporti_intestatario'
    cointestatario_rapporti_key = 'gestioni_rapporti_cointestatario'

    def dispatch(self, request, *args, **kwargs):
        self.instance_id = int(kwargs.pop('instance_id'))
        return super(GestioniStoricoEstrattoContoDownload, self).dispatch(request, *args, **kwargs)

    def get(self, request):
        file_gestioni = FileGestioni.objects.get(id=self.instance_id)
        rapporti_intestatario = ["0{}".format(rapp) for rapp in self.cliente[self.intestatario_rapporti_key]]
        rapporti_cointestatario = ["0{}".format(rapp) for rapp in self.cliente[self.cointestatario_rapporti_key]]
        rapporti_cliente = set(rapporti_intestatario+rapporti_cointestatario)
        if bool(len([rapp for rapp in rapporti_cliente if rapp in file_gestioni.rapporto])):
            response = HttpResponse(file_gestioni.file, content_type='application/pdf')
            response['Content-Disposition'] = 'attachment; filename={}'.format(file_gestioni.name)
            return response
        raise Http404


class GestioniRiclassificazionePortafogli(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/gestioni/riclassificazione_portafogli.html"
    prodotto = "GESTIONI"
    intestatario_rapporti_key = 'gestioni_contratti_intestatario'
    cointestatario_rapporti_key = 'gestioni_contratti_cointestatario'

    def get_context_data(self, **kwargs):
        context = super(GestioniRiclassificazionePortafogli, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))
        session_key = "riclassificazione_portafogli_{}_{}_{}".format(self.prodotto, self.partecipazione, rapporto)
        datasets = self.request.session.get(session_key, None)
        url_dettaglio = reverse(
            'myacomea:gestioni_riclassificazione_portafogli_dettaglio', kwargs={'partecipazione': self.partecipazione}
        )
        if self.is_cliente_full and self.codice_fiscale:
            if self.request.session.get("codice_fiscale", None) or not datasets:
                datasets = self.soap_client.get_AR_Riclassificazione_GI(
                    self.partecipazione, self.codice_fiscale, rapporto
                )
        elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
            self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
        elenco_rapporti = [rapp for rapp in elenco_rapporti]
        context.update({
            'partecipazione': self.partecipazione,
            'prodotto': self.prodotto,
            'url_dettaglio': url_dettaglio,
            'rapporti': elenco_rapporti,
            'datasets': datasets,
            'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
            'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
        })
        return context

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["my_acomea/includes/_dettaglio_riclassificazione_portafogli.html"]
        return super(GestioniRiclassificazionePortafogli, self).get_template_names()


class GestioniDettaglioTitoli(MyAcomeAMixin):
    template_name = "/my_acomea/prodotto/gestioni/dettaglio_titoli.html"
    prodotto = "GESTIONI"
    intestatario_rapporti_key = 'gestioni_contratti_intestatario'
    cointestatario_rapporti_key = 'gestioni_contratti_cointestatario'

    def get_context_data(self, **kwargs):
        context = super(GestioniDettaglioTitoli, self).get_context_data(**kwargs)
        rapporto = int(self.request.GET.get('rapporto', 0))
        session_key = "dettaglio_titoli_{}_{}_{}".format(self.prodotto, self.partecipazione, rapporto)
        titoli = self.request.session.get(session_key, None)
        url_dettaglio = reverse(
            'myacomea:gestioni_dettaglio_titoli_dettaglio', kwargs={'partecipazione': self.partecipazione}
        )
        if self.is_cliente_full and self.codice_fiscale:
            if self.request.session.get("codice_fiscale", None) or not titoli:
                titoli = self.soap_client.get_AR_Titoli_GI(self.partecipazione, self.codice_fiscale, rapporto)
        elenco_rapporti = self.cliente[self.intestatario_rapporti_key] if \
            self.partecipazione.lower() == "intestatario" else self.cliente[self.cointestatario_rapporti_key]
        elenco_rapporti = [rapp for rapp in elenco_rapporti]
        context.update({
            'partecipazione': self.partecipazione,
            'prodotto': self.prodotto,
            'url_dettaglio': url_dettaglio,
            'rapporti': elenco_rapporti,
            'titoli': titoli,
            'is_intestatario': bool(len(self.cliente[self.intestatario_rapporti_key])),
            'is_cointestatario': bool(len(self.cliente[self.cointestatario_rapporti_key])),
            'modal_url': reverse('myacomea:gestioni_dettaglio_titoli_modale'),
            'modal_id': "GenericModal",
        })
        return context

    def get_template_names(self):
        if 'rapporto' in self.request.GET:
            return ["my_acomea/includes/_dettaglio_gestioni_dettaglio_titoli.html"]
        return super(GestioniDettaglioTitoli, self).get_template_names()


class GestioniDettaglioTitoliModale(TemplateView):
    template_name = "/my_acomea/prodotto/gestioni/dettaglio_titoli_modale.html"
    soap_client = MyAcomeaSoapClient()

    def get_context_data(self, **kwargs):
        context = super(GestioniDettaglioTitoliModale, self).get_context_data(**kwargs)
        codice_titolo = self.request.GET.get('codice-titolo')
        rapporto = self.request.GET.get('rapporto')
        dettaglio = self.soap_client.get_AR_TitoliDettPrz_GI(codice_titolo, rapporto)
        context.update({'dettaglio': dettaglio})
        return context


############################################################################
# EXPORT ###################################################################
############################################################################
class FondiDettaglioExport(TemplateView):
    response_class = OdtTemplateResponse
    template_name = "my_acomea/odt/test.odt"
    session_keys = ["dettaglio_fondi_bin_data"]
    session_image_keys = ["dettaglio_fondi_bin_data"]

    def get_context_data(self, **kwargs):
        context = super(FondiDettaglioExport, self).get_context_data(**kwargs)
        context.update({
            'prova': 'Ciao',
            'dettaglio_fondi_bin_data': get_photo_appy(
                self.create_b64_image(self.request.session['dettaglio_fondi_bin_data'])
            )
        })
        """
        for key in self.session_keys:
            if key in self.request.session and self.request.session[key]:
                value = self.request.session[key]
                context.update({key: value if key not in self.session_image_keys else self.create_b64_image(value)})
        """
        return context

    @staticmethod
    def create_b64_image(b64_image, filename='prova.png'):
        output_filename = os.path.join(settings.MEDIA_ROOT, 'highcharts', "{}".format(filename))
        data = b64_image.replace(' ', '+')
        png_recovered = base64.decodestring(data)
        chart_file = open(output_filename, "w")
        chart_file.write(png_recovered)
        chart_file.close()
        return output_filename
